/*
 * Sonar PL/SQL Toad Plugin
 * Copyright (C) 2012 SonarSource
 * dev@sonar.codehaus.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.plsqltoad;

import com.google.common.base.Preconditions;
import com.google.common.base.Throwables;
import org.apache.commons.io.FileUtils;
import org.sonar.api.BatchExtension;
import org.sonar.api.ServerExtension;
import org.sonar.api.config.Settings;
import org.sonar.api.resources.Project;

import javax.annotation.Nullable;

import java.io.File;
import java.io.IOException;

import static com.google.common.base.Preconditions.checkNotNull;

public class PlSqlToadPluginConfiguration implements ServerExtension, BatchExtension {

  private final Settings settings;
  private final Project project;

  private File toadOutputDirectory = null;

  public PlSqlToadPluginConfiguration(Settings settings) {
    this(settings, null);
  }

  public PlSqlToadPluginConfiguration(Settings settings, @Nullable Project project) {
    this.settings = settings;
    this.project = project;
  }

  public synchronized File getToadOutputDirectory() {
    checkNotNull(project, "This method is not available from the server side");

    if (toadOutputDirectory == null) {
      try {
        toadOutputDirectory = new File(project.getFileSystem().getSonarWorkingDirectory(), "toad");

        FileUtils.forceMkdir(toadOutputDirectory);
        FileUtils.cleanDirectory(toadOutputDirectory);
      } catch (IOException e) {
        Throwables.propagate(e);
      }
    }

    return toadOutputDirectory;
  }

  public String getToadExePath() {
    String value = null;

    if (!settings.hasKey(PlSqlToadPlugin.TOAD_EXE_PATH_KEY) && settings.hasKey(PlSqlToadPlugin.TOAD_EXE_PATH_OLD_KEY)) {
      value = settings.getString(PlSqlToadPlugin.TOAD_EXE_PATH_OLD_KEY);
    }

    if (value == null) {
      value = settings.getString(PlSqlToadPlugin.TOAD_EXE_PATH_KEY);
    }

    if (value == null) {
      return "";
    }

    Preconditions.checkArgument(new File(value).isFile(), "The Toad executable path property \"" + value + "\" does not point to a valid file");
    return value;
  }

  public String getToad10Login() {
    if (!settings.hasKey(PlSqlToadPlugin.TOAD10_LOGIN_KEY) && settings.hasKey(PlSqlToadPlugin.TOAD10_LOGIN_OLD_KEY)) {
      return settings.getString(PlSqlToadPlugin.TOAD10_LOGIN_OLD_KEY);
    }

    String value = settings.getString(PlSqlToadPlugin.TOAD10_LOGIN_KEY);

    return value != null ? value : "";
  }

  public String getToad10Password() {
    if (!settings.hasKey(PlSqlToadPlugin.TOAD10_PASSWORD_KEY) && settings.hasKey(PlSqlToadPlugin.TOAD10_PASSWORD_OLD_KEY)) {
      return settings.getString(PlSqlToadPlugin.TOAD10_PASSWORD_OLD_KEY);
    }

    String value = settings.getString(PlSqlToadPlugin.TOAD10_PASSWORD_KEY);

    return value != null ? value : "";
  }

  public boolean hasToad10Credentials() {
    return !"".equals(getToad10Login());
  }

  public int getToadTimeout() {
    if (!settings.hasKey(PlSqlToadPlugin.TOAD_TIMEOUT_KEY) && settings.hasKey(PlSqlToadPlugin.TOAD_TIMEOUT_OLD_KEY)) {
      return settings.getInt(PlSqlToadPlugin.TOAD_TIMEOUT_OLD_KEY);
    }

    return settings.getInt(PlSqlToadPlugin.TOAD_TIMEOUT_KEY);
  }

  public int getToadLimit() {
    if (!settings.hasKey(PlSqlToadPlugin.TOAD_LIMIT_KEY) && settings.hasKey(PlSqlToadPlugin.TOAD_LIMIT_OLD_KEY)) {
      return settings.getInt(PlSqlToadPlugin.TOAD_LIMIT_OLD_KEY);
    }

    return settings.getInt(PlSqlToadPlugin.TOAD_LIMIT_KEY);
  }

}
