/*
 * Sonar PL/SQL Toad Plugin
 * Copyright (C) 2012 SonarSource
 * dev@sonar.codehaus.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.plsqltoad;

import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonar.api.batch.Sensor;
import org.sonar.api.batch.SensorContext;
import org.sonar.api.profiles.RulesProfile;
import org.sonar.api.resources.InputFile;
import org.sonar.api.resources.Project;
import org.sonar.api.utils.command.Command;
import org.sonar.api.utils.command.CommandExecutor;

import java.util.List;

public class PlSqlToadSensor implements Sensor {

  private static final Logger LOG = LoggerFactory.getLogger(PlSqlToadSensor.class);

  private final PlSqlToadPluginConfiguration pluginConf;
  private final RulesProfile profile;

  public PlSqlToadSensor(PlSqlToadPluginConfiguration pluginConf, RulesProfile profile) {
    this.pluginConf = pluginConf;
    this.profile = profile;
  }

  public boolean shouldExecuteOnProject(Project project) {
    return PlSqlToadPlugin.LANGUAGE_KEY.equals(project.getLanguageKey());
  }

  public void analyse(Project project, SensorContext context) {
    if (!shouldExecuteToad(project)) {
      return;
    }

    List<PlSqlToadPage> pages = PlSqlToadPageBuilder.generate(getPlSqlFilesToAnalyze(project), pluginConf, profile);
    executeToad(pages);
    collectMeasures(project, context, pages);
  }

  public List<java.io.File> getPlSqlFilesToAnalyze(Project project) {
    List<java.io.File> plSqlFiles = Lists.newArrayList();

    for (InputFile file : project.getFileSystem().mainFiles(PlSqlToadPlugin.LANGUAGE_KEY)) {
      plSqlFiles.add(file.getFile());
    }

    return plSqlFiles;
  }

  private boolean isNoToadRuleActive() {
    return profile.getActiveRulesByRepository(PlSqlToadPlugin.KEY_REPOSITORY_TOAD).size() == 0
      && profile.getActiveRulesByRepository(PlSqlToadPlugin.KEY_REPOSITORY_TOAD_11).size() == 0;
  }

  private boolean shouldExecuteToad(Project project) {
    if (getPlSqlFilesToAnalyze(project).isEmpty()) {
      /* Not done in shouldExecuteOnProject() because new files might get extracted from Forms */
      return false;
    }

    if (isNoToadRuleActive()) {
      LOG.info("Skipping the execution of PL/SQL's Toad engine because no Toad rule is enabled in the current profile.");
      return false;
    }

    return true;
  }

  private void executeToad(List<PlSqlToadPage> pages) {
    LOG.info("There are {} planned executions of Toad.", pages.size());

    long start = System.currentTimeMillis();

    int currentPage = 1;
    for (PlSqlToadPage page : pages) {
      LOG.info("Starting the Toad execution {} out of {}..", currentPage, pages.size());
      executeToad(page);
      currentPage++;
    }

    LOG.info("All {} Toad executions completed in {} ms", pages.size(), System.currentTimeMillis() - start);
  }

  protected void executeToad(PlSqlToadPage page) {
    CommandExecutor executor = CommandExecutor.create();
    Command command = Command.create(pluginConf.getToadExePath()).addArgument("-CX")
        .addArgument(page.getIniFile().getAbsolutePath());
    int exitCode = executor.execute(command, pluginConf.getToadTimeout() * 1000L);
    if (exitCode != 0) {
      throw new IllegalStateException("The exit code was " + exitCode + " when 0 was expected for command: " + command);
    }
  }

  private void collectMeasures(Project project, SensorContext context, List<PlSqlToadPage> pages) {
    long start = System.currentTimeMillis();

    PlSqlToadResultsParser parser = new PlSqlToadResultsParser(project, context, profile);
    for (PlSqlToadPage page : pages) {
      parser.parse(page);
    }

    LOG.info("Toad results analysed in {} ms", System.currentTimeMillis() - start);
  }

}
